function [hat_A,hat_X] = NMF_APC_algorithm(D,r)
% algorithm for non-negative matrix factorization D = AX',
% with sparsity and smoothness model on X and sparse model on A

N = 100; % number of iteration

[p,n]=size(D);
nr = n*r;

% initialization
psi0 = 1e-10*ones(r,p);
fi0 = 1e-10*ones(r,p);
hat_AtA = 1e-10*ones(r,r);
hat_xi = ones(r,p);
hat_omega = 1;
hat_A = 1e-10*ones(p,r);
hat_a = hat_A';

l0 = -0.7*ones(nr-1,1);
for t = 1:r-1
    q = n*t;
    l0(q) = 0;
end
hat_l = l0;
hat_ll = ones(nr-1,nr-1);
hat_v = ones(nr,1);
sigma_l = 1e-10*ones(nr-1,1);
mi_l = l0;
hat_ofi = ones(nr-1,1);

% define shapes
hat_aaT = zeros(r,r,p);
SIGMA_a = zeros(r,r,p);
mi_a = zeros(r,p);
psi = zeros(r,p);

% define priors
vartheta0 = 1e-10;
ro0 = 1e-10;
alpha0 = 1e-10;
beta0 = 1e-10;
c0 = 1e-2;
eta0 = 1e-2;

% pre-calculation
vartheta = vartheta0+(p*n)/2;
fi = fi0+1/2*ones(r,p);
alpha = alpha0+1/2*ones(nr,1);
c = c0 +1/2*ones(nr-1,1);
pom = D'*D;
%%
for l = 1:N
    if mod(l,10) == 0
        display(['NMF-APC: iteration ' num2str(l) '/' num2str(N)])    
    end

    % LVL'
    LYL_1 = diag(hat_v(1:end-1).*hat_l,-1);
    LYL_2 = diag([hat_v(1);hat_v(1:end-1).*diag(hat_ll)+hat_v(2:end)]);
    LYL_3 = diag(hat_v(1:end-1).*hat_l,1);
    hat_LYL= LYL_1+LYL_2+LYL_3;

    % X
    DTA = D'*hat_a';
    VSIGMA_x = inv(hat_omega*kron(hat_AtA,eye(n,n))+hat_LYL);
    Vmi_x = VSIGMA_x*hat_omega*DTA(:);
    
    [Vhat_x,~,hat_xxT] = momenty_tNv2(Vmi_x,diag(VSIGMA_x),nr,1);
    hat_x = (reshape(Vhat_x,n,r))';
    [hat_XtX] = vypocetXtX(n,r,hat_xxT);
    
    % A
    for i=1:p
        SIGMA_a(:,:,i) = inv(hat_omega*hat_XtX++diag(hat_xi(:,i)));
        mi_a(:,i) = SIGMA_a(:,:,i)*hat_omega*sum(repmat(D(i,:),r,1).*hat_x,2);
        [hat_a(:,i),~,hat_aaT(:,:,i)] = momenty_tNv2(mi_a(:,i),diag(SIGMA_a(:,:,i)),r,1);
    end
    hat_AtA = sum(hat_aaT,3);
    
    % L
    for s = 1:nr-1
        sigma_l(s) = inv(hat_v(s)*hat_xxT(s+1,s+1)+hat_ofi(s));
        mi_l(s) = sigma_l(s)*(-hat_v(s)*hat_xxT(s,s+1)+l0(s)*hat_ofi(s));
        if 1
            if mod(s,n)==0
                sigma_l(s) = 0;
                mi_l(s) = 0;
            end
        end
    end
    SIGMA_l = diag(sigma_l);
    [hat_l,~,hat_llT] = momenty_tNv2(mi_l,diag(SIGMA_l),nr-1,1,-inf,inf);
    
    % omega
    ro = ro0 + 1/2*(trace(pom-2*hat_x'*hat_a*D)+trace(hat_XtX*hat_AtA));
    hat_omega = vartheta/ro;
    
    % xi
    for i = 1:p
       psi(:,i) = psi0(:,i)+1/2*diag(hat_aaT(:,:,i));
       hat_xi(:,i) = fi(:,i)./psi(:,i);
    end
    
    % v
    diagLxxL = [diag(hat_xxT(1:nr-1,1:nr-1))+2*diag(hat_xxT,1).*hat_l+diag(hat_xxT(2:nr,2:nr)).*diag(hat_llT);hat_xxT(nr,nr)];
    beta = beta0+1/2*diagLxxL;
    hat_v = alpha./beta;
    
    % fi
    eta = eta0+1/2*diag(hat_llT)-diag(hat_l*l0')+1/2*diag(l0*l0');
    hat_ofi = c./eta;
end

% return
hat_A = hat_a';
hat_X = hat_x';
end


%% auxiliary functions
function [hat_A,hat_AtA,hat_AAt] = momenty_tNv2(vec_mia,diag_sigmaA,p,r,a,b)
    % moments of truncated normal distribution
    if nargin < 7
        b = inf;
        if nargin < 6
            a = 0;
        end
    end
    diag_sigmaA(diag_sigmaA < 0) = 1e-10;
    pom = sqrt(2*diag_sigmaA);
    alphaa = (a-vec_mia)./pom; 
    betaa = (b-vec_mia)./pom; 
    TTB = b*exp(-betaa.^2);
    TTA = a*exp(-alphaa.^2);
    if b == inf
        TTB = 0;
    end
    if a == -inf
        TTA = 0;
    end
    % stability
    istab = find(alphaa <= 3); % stable
    astab = find(alphaa > 3); % unstable

    vhat_a = zeros(size(vec_mia));
    vhat_ata = zeros(size(vec_mia));
    if ~isempty(istab)
        pom1 = sqrt(diag_sigmaA(istab));
        pom2 = erf(betaa(istab));
        pom3 = erf(alphaa(istab));
        pom4 = exp(-alphaa(istab).^2);
        pom5 = (sqrt(pi)*(pom2 - pom3));

        vhat_a(istab) = vec_mia(istab) - pom1.*(sqrt(2)*...
            (exp(-betaa(istab).^2) - pom4)./pom5);
       if  a == -inf
        vhat_ata(istab) = diag_sigmaA(istab) + vec_mia(istab).*vhat_a(istab) - ...
            pom1.*(sqrt(2)*(TTB - TTA)./pom5);
       else
        vhat_ata(istab) = diag_sigmaA(istab) + vec_mia(istab).*vhat_a(istab) - ...
            pom1.*(sqrt(2)*(TTB - TTA(istab))./pom5);
       end
    end
    if ~isempty(astab)
        vhat_a(astab) = a - sqrt(diag_sigmaA(astab)).^2./vec_mia(astab);
        vhat_ata(astab) = 0;
    end
    % matrices
    hat_A = reshape(vhat_a,p,r);
    hat_AtA = diag((reshape(vhat_ata,p,r)')*ones(p,1))+...
        (ones(r)-eye(r)).*(hat_A'*hat_A);
    if nargout > 2
        if p > 1000 % too large
            hat_AAt = NaN;
            display('pozor, momenty_tN siri NaN!!')
        else
            hat_AAt = diag((reshape(vhat_ata,p,r))*ones(r,1))+...
                (ones(p)-eye(p)).*(hat_A*hat_A');
        end
    end
end

function [hat_XtX] = vypocetXtX(n,r,hat_xxT)
    % moment of the X'X type
    diagxxT = diag(hat_xxT);
    diagXtX = zeros(r,1);
    % diagonal XtX
    konst = 1;
    for indi = 1:r
        diagXtX(indi) = sum(diagxxT(konst:konst+(n-1)));
        konst = konst + n;
    end

    diagXtXvedlajsie = zeros(r-1,r-1);
    % sub-diagonals
    for indj = 1:r-1
        diagxxTvedlajsie = diag(hat_xxT,-n*indj);
        konst = 1;
        for indi = 1:r-indj
            diagXtXvedlajsie(indi,indj) = sum(diagxxTvedlajsie(konst:konst+(n-1)));
            konst = konst + n;
        end
    end

    % XtX
    hat_XtX = diag(diagXtX);
    konst = r-1;
    for indj = 1:r-1
        diag_pod = diag(diagXtXvedlajsie(1:konst,indj),-indj);
        diag_nad = diag(diagXtXvedlajsie(1:konst,indj),indj);
        hat_XtX = hat_XtX + diag_pod + diag_nad;
        konst = konst - 1;
    end
end



